@@ -20,6 +20,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:freetype-fonts
+ * @short_description:Functions for shape engines to manipulate FreeType fonts
+ * @title:FreeType Fonts and Rendering
+ *
+ * The macros and functions in this section are used to access fonts and render
+ * text to bitmaps using the FreeType 2 library.
+ */
 #include "config.h"
 
 #include <string.h>
@@ -102,7 +110,7 @@ load_fallback_face (PangoFT2Font *ft2font,
 
   _pango_ft2_font_map_default_substitute ((PangoFcFontMap *)fcfont->fontmap, sans);
 
-  matched = FcFontMatch (NULL, sans, &result);
+  matched = FcFontMatch (pango_fc_font_map_get_config ((PangoFcFontMap *)fcfont->fontmap), sans, &result);
 
   if (FcPatternGetString (matched, FC_FILE, 0, &filename2) != FcResultMatch)
     goto bail1;
@@ -161,8 +169,9 @@ set_transform (PangoFT2Font *ft2font)
  * face from pango_fc_font_lock_face() you must call
  * pango_fc_font_unlock_face().
  *
- * Return value: a pointer to a <type>FT_Face</type> structure, with the size set correctly,
- *               or %NULL if @font is %NULL.
+ * Return value: (nullable): a pointer to a <type>FT_Face</type>
+ *               structure, with the size set correctly, or %NULL if
+ *               @font is %NULL.
  **/
 FT_Face
 pango_ft2_font_get_face (PangoFont *font)
@@ -467,12 +476,13 @@ pango_ft2_font_finalize (GObject *object)
 
 /**
  * pango_ft2_font_get_coverage:
- * @font: a #PangoFT2Font.
+ * @font: a <type>PangoFT2Font</type>.
  * @language: a language tag.
- * @returns: a #PangoCoverage.
  *
- * Gets the #PangoCoverage for a #PangoFT2Font. Use
+ * Gets the #PangoCoverage for a <type>PangoFT2Font</type>. Use
  * pango_font_get_coverage() instead.
+ *
+ * Return value: a #PangoCoverage.
  **/
 PangoCoverage *
 pango_ft2_font_get_coverage (PangoFont     *font,
@@ -507,51 +517,6 @@ pango_ft2_get_unknown_glyph (PangoFont *font)
     return PANGO_GLYPH_EMPTY;
 }
 
-typedef struct
-{
-  FT_Error     code;
-  const char   msg[40];
-} ft_error_description;
-
-static int
-ft_error_compare (const void *pkey,
-		  const void *pbase)
-{
-  return ((ft_error_description *) pkey)->code - ((ft_error_description *) pbase)->code;
-}
-
-const char *
-_pango_ft2_ft_strerror (FT_Error error)
-{
-#undef __FTERRORS_H__
-#define FT_ERRORDEF( e, v, s )  { e, s },
-#define FT_ERROR_START_LIST  {
-#define FT_ERROR_END_LIST    };
-
-  static const ft_error_description ft_errors[] =
-#include FT_ERRORS_H
-
-#undef FT_ERRORDEF
-#undef FT_ERROR_START_LIST
-#undef FT_ERROR_END_LIST
-
-  ft_error_description *found =
-    bsearch (&error, ft_errors, G_N_ELEMENTS (ft_errors),
-	     sizeof (ft_errors[0]), ft_error_compare);
-  if (found != NULL)
-    return found->msg;
-  else
-    {
-      static char *default_msg = NULL; /* MT-safe */
-
-      if (g_once_init_enter (&default_msg))
-	g_once_init_leave (&default_msg, g_malloc (60));
-
-      g_sprintf (default_msg, "Unknown FreeType2 error %#x", error);
-      return default_msg;
-    }
-}
-
 void *
 _pango_ft2_font_get_cache_glyph_data (PangoFont *font,
 				     int        glyph_index)
